<?php

	namespace aidSQL\db{

		abstract class AbstractDbBuilder{

			protected	$_config			=	NULL;
			protected	$_logger			=	NULL;
			protected	$_httpAdapter	=	NULL;
			protected	$_dbAdapter		=	NULL;
			protected	$_pLoader		=	NULL;
			protected	$_xmlParser		=	NULL;

			protected	$_schemas		=	NULL;
			protected	$_injection		=	NULL;
			protected	$_tablePrefix	=	NULL;
			protected	$_schemaPrefix	=	NULL;

			final public function __construct(Array $config,\aidSQL\parser\DbXML &$xmlParser,\aidSQL\core\Logger &$log){

				$this->setConfig($config);
				$this->setXmlParser($xmlParser);
				$this->setLog($log);
				$schemas	=	$xmlParser->getSchemas();

				if(!$schemas){

					throw(new \Exception("COULDN'T GET ANY SCHEMAS TO BE REVERSED FROM THIS XML FILE!"));

				}

				$this->setSchemas($schemas);
				$this->setInjectionParameters($xmlParser->getInjectionParameters());


			}

			public function setHttpAdapter(\aidSQL\http\adapter &$httpAdapter){

				//Parse vulnerable URI

				$injection	=	$this->_xmlParser->getInjectionParameters();
				$httpMethod	=	$this->_xmlParser->getHttpMethod();

				$httpAdapter->setMethod($httpMethod);

				$url			=	new \aidSQL\core\Url($this->_xmlParser->getVulnerableUri());
				$url->addRequestVariable($injection["requestVariable"],$injection["requestValue"]);

				if(isset($injection["requestVariables"])){

					foreach($injection["requestVariables"] as $name=>$value){

						$url->addRequestVariable($name,$value);

					}

				}


				if($this->_config["verbose"]){

					$httpAdapter->setVerbose($this->_config["verbose"]);

				}

				$httpAdapter->setUrl($url);
				
				$this->_httpAdapter	=	$httpAdapter;

			}

			public function getHttpAdapter(){

				return $this->_httpAdapter;

			}

			public function setDbAdapter(\aidSQL\db\MySQLDbAdapter $dbAdapter){

				$this->_dbAdapter	=	$dbAdapter;

			}

			public function setPluginLoader(\aidSQL\core\PluginLoader &$pLoader){

				$this->_pLoader	=	$pLoader;

			}

			public function getPluginLoader(){

				return $this->_pLoader;

			}

			public function setConfig(Array $config){

				$this->_config	=	$config;

			}

			public function getConfig(){

				return $this->_config;

			}

			public function setXmlParser(\aidSQL\parser\DbXML $parser){

				$this->_xmlParser	=	$parser;

			}

			public function setLog(\aidSQL\core\Logger &$logger){

				$logger->setPrepend('['.__CLASS__.']');
				$this->_logger	=	$logger;

			}


			protected function log($msg=NULL, $color="white", $level=0) {

				if (isset($this->_config["log-all"])) {
					$toFile = TRUE;
				}

				if (!is_null($this->_logger)) {

					$this->_logger->setPrepend('[' . __CLASS__ . ']');
					$this->_logger->log($msg, $color, $level);
					return TRUE;

				}

				return FALSE;

			}

			public function setSchemaPrefix($prefix){

				$this->_schemaPrefix	=	$prefix;

			}

			public function getSchemaPrefix(){

				return $this->_schemaPrefix;

			}

			public function setTablePrefix($tablePrefix){

				$this->_tablePrefix	=	$tablePrefix;

			}

			public function getTablePrefix(){

				return $this->_tablePrefix;

			}


			public function createDatabase($schemaName){

				if(is_null($this->_schemaPrefix)){

					$host				=	$this->_xmlParser->getVulnerableUri();
					$host				=	new \aidSQL\core\Url($host);
					$host				=	preg_replace("#\W#",'_',$host->getHost());
					$schemaPrefix	=	"aidSQL_".$host.'_';

					unset($host);

					$this->log("Assuming default schema prefix $schemaPrefix",2,"light_purple");
					$this->setSchemaPrefix($schemaPrefix);
					
				}

				$schemaName	=	$this->_schemaPrefix.$this->_dbAdapter->real_escape_string($schemaName);

				$sql	=	"CREATE DATABASE IF NOT EXISTS $schemaName";

				$create	=	$this->_dbAdapter->query($sql);
				$select	=	$this->_dbAdapter->select_db($schemaName);


				return $create & $select;

			}

			public function createTable($tableName,Array $tableColumns){

				$tableName	=	$this->_tablePrefix.$this->_dbAdapter->real_escape_string($tableName);
				$sql			=	NULL;
				$columns		=	Array();

				foreach($tableColumns["columns"] as $columnName=>$columnSpecs){

					//$columns[]	=	$this->_dbAdapter->real_escape_string($columnName).' '.
					//					$this->_dbAdapter->real_escape_string($columnSpecs["type"]);

					$columns[]	=	$columnName.' '.
										$columnSpecs["type"];

				}

				//FIX
				foreach($columns as &$column){

					if(preg_match('/\([0-9]+$/',$column)&&substr($column,-1)!=')'){

						$column.=')';

					}

				}

				$columns		=	implode(',',$columns);
				$sql			=	"CREATE TABLE IF NOT EXISTS $tableName(".$columns.")";
				$truncate	=	"TRUNCATE TABLE $tableName";
				
				if(!($this->_dbAdapter->query($sql) && $this->_dbAdapter->query($truncate))!==TRUE){

					if($this->_dbAdapter->errno>0){

						$msg	=	"Couldnt create table $tableName ".$this->_dbAdapter->errno.':'.$this->_dbAdapter->error;

						throw(new \Exception($msg));

					}

					return FALSE;

				}

				return TRUE;

			}

			public function insertRegisters($tableName,Array $columns,Array $registers){

				$tableName	=	$this->_dbAdapter->real_escape_string($tableName);			
				$tableName	=	$this->getTablePrefix().$tableName;

				$sql		=	"INSERT INTO $tableName SET ";

				foreach($registers as $k=>$reg){

					$registers[$k]	=	$this->_dbAdapter->real_escape_string($reg);

				}

				$result	=	@array_combine($columns,$registers);

				if(!$result){

					//FIX ME
					$registers	=	array_pad($registers,sizeof($columns),'-');
					$result		=	array_combine($columns,$registers);
					$this->log("WARNING! INSERTING PADDED VALUES, THIS DATA IS NOT ACCURATE!",1,"red");

				}

				foreach($result as $colName=>$colVal){

					$sql.="$colName='$colVal',";

				}

				$sql=substr($sql,0,-1);

				return $this->_dbAdapter->query($sql);

			}

			public function setInjectionParameters(Array $injectionParameters){

				$this->_injection			=	$injectionParameters;

			}

			public function setSchemas(Array $schemas){

				if(!sizeof($schemas)){

					throw(new \Exception("Error! You can't specify an empty schemas Array! :/"));
					return; 

				}

				if(array_key_exists("interactive",$this->_config)){

					foreach($schemas as $schemaName=>$schemaTables){

						$this->_logger->log("REVERSE ENGINEER $schemaName?",0,"white");

						$selectedIndexes	=	interactive($this->_logger,Array("YES","NO"),NULL,FALSE,TRUE);

						if($selectedIndexes===TRUE){

							$this->log("Will reverse engineer ALL schemas with ALL tables!",0,"yellow");
							break;

						}


						if($selectedIndexes[0]==1){

							unset($schemas[$schemaName]);
							$this->log("Skipping schema \"$schemaName\"",0,"yellow");
							continue;

						}

						if(!sizeof($schemaTables)){

							$this->_logger->log("WELL, NO TABLES FOUND IN THIS SCHEMA :/",0,"light_purple");
							unset($schemas[$schemaName]);
							continue;

						}

						$this->_logger->log("SELECT WHICH TABLES FROM \"$schemaName\" YOU WANT TO REVERSE ENGINEER",0,"white");
						$this->_logger->log("TIP:You can specify multiple tables by using ','. For instance 1,2,3",0,"white");

						$tableIndexes		=	array_keys($schemaTables);
						$selectedIndexes	=	interactive($this->_logger,$tableIndexes,NULL,FALSE);


						if(!($selectedIndexes===TRUE)){

							$tmpTables			=	Array();

							foreach($schemaTables as $tableName=>$values){

								$values["tableName"]	=	$tableName;
								$tmpTables[]			=	$values;

							}

							$tmpTables		=	removeNonSelectedOptions($tmpTables,$selectedIndexes);

							$newTables		=	Array();	

							foreach($tmpTables as $index=>$values){

								$newTables[$values["tableName"]]	=	$values;

							}

							$schemaTables	=	$newTables;

							foreach($schemaTables as $tableName=>$values){

								$this->log("Adding table $tableName ...",0,"light_green");

							}

							$schemas[$schemaName]	=	$schemaTables;

						}else{

							$this->log("Adding ALL tables!",0,"light_green");
							
						}


					}

				}

				if(!sizeof($schemas)){

					throw(new \Exception("No schemas left to be reversed :/"));
					return; 

				}

				$this->_schemas	=	$schemas;

			}

			abstract public function reverse();

		}

	}

?>
